#ifndef __GraphSimplifier_h__
#define __GraphSimplifier_h__

#include "Error.h"
#include "Stringf.h"

class GraphSimplifier
{
public:
    void simplify( const String &programFile )
    {
        renameProgram( programFile );

        File input( _oldProgramFile.ascii() );
        File output( _newProgramFile.ascii() );

        input.open( IFile::MODE_READ );
        output.open( IFile::MODE_WRITE );

        String line;

        while ( ( line = input.readLine() ).length() > 0 )
        {
            handleInputLine( output, line );
        }

        removeTempFile();
    }

private:
    String _oldProgramFile;
    String _newProgramFile;

    void renameProgram( String programFile )
    {
        _oldProgramFile = programFile + ".original";
        _newProgramFile = programFile;

        if ( rename( _newProgramFile.ascii(), _oldProgramFile.ascii() ) == -1 )
        {
            throw Error( Error::RENAME_FAILED );
        }
    }

    void removeTempFile()
    {
        if ( unlink( _oldProgramFile.ascii() ) == -1 )
        {
            throw Error( Error::UNLINK_FAILED );
        }
    }

    void handleInputLine( File &output, String &line )
    {
        if ( alphabetLine( line ) )
        {
            if ( eventLine( line ) )
            {
                output.write( line + "\n" );
            }
        }
        else if ( transitionLine( line ) )
        {
            output.write( String( "\t\t<Label>" ) + onlyKeepEvents( line ) + "</Label>\n" );
        }
        else
        {
            output.write( line + "\n" );
        }
    }

    bool eventLine( String line )
    {
        return
            ( line[ line.find( "<Proposition>" ) + strlen( "<Proposition>" ) ] != 'b' ) &&
            ( line[ line.find( "<Proposition>" ) + strlen( "<Proposition>" ) ] != 'r' );
    }

    bool transitionLine( const String &line ) const
    {
        return line.contains( "<Label>" );
    }

    bool alphabetLine( const String &line ) const
    {
        return line.contains( "<Proposition>" ) && line.contains( "</Proposition>" );
    }

    String onlyKeepEvents( String line )
    {
        String result;
        Vector<String> tokens = ( stripLabelTags( line ) ).tokenize();

        for ( unsigned i = 0; i < tokens.size(); ++i )
        {
            String token = tokens[i];

            if ( !token.contains( "r" ) && !token.contains( "b" ) )
            {
                result += token + " ";
            }
        }

        return result;
    }

    String stripLabelTags( String input )
    {
        unsigned start = input.find( "<Label>" ) + strlen( "<Label>" );
        unsigned howMany = input.find( "</Label>" ) - start;

        return input.substring( start, howMany );
    }
};

#endif // __GraphSimplifier_h__

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
