#ifndef __AutomaticKeyPad_h__
#define __AutomaticKeyPad_h__

#include "BThread.h"
#include "IEvent.h"
#include "String.h"
#include "Stringf.h"
#include "T/Truth.h"

class AutomaticKeyPad : public BThread
{
public:
    AutomaticKeyPad() : BThread( "KeyPad" ), _numberOfRequests( 0 ), _totalDelay( 0 )
    {
    }

    void entryPoint()
    {
        while ( T::alwaysTrue() )
        {
            sleepUntilNextRequest();
            makeRequest();
        }
    }
private:
    Vector<Event> _requested;
    Vector<Event> _watched;
    Vector<Event> _blocked;

    time_t _requestTime;

    unsigned _numberOfRequests;
    unsigned _totalDelay;

    static time_t now()
    {
        return time( NULL );
    }

    void clearAll()
    {
        _requested.clear();
        _watched.clear();
        _blocked.clear();
    }

    void sleepUntilNextRequest()
    {
        enum {
            MAX_TIME_UNTIL_NEXT_REQUEST = 5,
        };

        sleep( rand() % MAX_TIME_UNTIL_NEXT_REQUEST );
    }

    void makeRequest()
    {
        takeTime();
        insertCoin();
        requestProduct();
        waitForProduct();
        updateStatistics();
    }

    void takeTime()
    {
        _requestTime = now();
    }

    void insertCoin()
    {
        clearAll();
        _requested.append( IEvent::COIN_INSERTED );
        bSync( _requested, _watched, _blocked, "Coin" );
    }

    void requestProduct()
    {
        ++_numberOfRequests;

        clearAll();
        _requested.append( IEvent::PRODUCT_CHOSEN );
        bSync( _requested, _watched, _blocked, "Chosen" );
    }

    void waitForProduct()
    {
        clearAll();
        _watched.append( IEvent::PRODUCT_PROVIDED );
        bSync( _requested, _watched, _blocked, "Received" );
    }

    void updateStatistics()
    {
        time_t receivedTime = now();
        unsigned elapsedTime = receivedTime - _requestTime;
        _totalDelay += elapsedTime;

        printf( "This round's delay: %u\n", elapsedTime );

        if ( _numberOfRequests % 10 == 0 )
        {
            printf( "\nNumber of products: %u. Average delay: %f\n\n", _numberOfRequests, (float)(_totalDelay) / _numberOfRequests );
            fflush( 0 );
        }

        if ( _numberOfRequests == 250 )
        {
            print( "DONE" );
            exit( 0 );
        }
    }

    void print( String line )
    {
        printf( "%s\t%s\n", timeString().ascii(), line.ascii() );
    }

    String timeString()
    {
        time_t timeSinceEpoch = time( NULL );

        struct tm *localTime = localtime( &timeSinceEpoch );

        return Stringf( "%0.2d:%0.2d:%0.2d", localTime->tm_hour, localTime->tm_min, localTime->tm_sec );
    }

};

#endif // __AutomaticKeyPad_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
