#ifndef __Dispenser_h__
#define __Dispenser_h__

#include "BThread.h"
#include "IEvent.h"
#include "Selector.h"
#include "ServerSocket.h"
#include "Socket.h"
#include "String.h"
#include "T/Truth.h"

class Dispenser : public BThread
{
public:
    Dispenser() : BThread( "Dispenser" )
    {
    }

    void entryPoint()
    {
        while ( T::alwaysTrue() )
        {
            waitForCoin();
            waitForProductSelection();
            provideProduct();
        }
    }

private:
    Vector<Event> _requested;
    Vector<Event> _watched;
    Vector<Event> _blocked;

    void clearAll()
    {
        _requested.clear();
        _watched.clear();
        _blocked.clear();
    }

    void waitForCoin()
    {
        clearAll();
        _watched.append( IEvent::COIN_INSERTED );
        bSync( _requested, _watched, _blocked, "Wait for coin" );
    }

    void waitForProductSelection()
    {
        clearAll();
        _watched.append( IEvent::PRODUCT_CHOSEN );
        bSync( _requested, _watched, _blocked, "Wait for product selection" );
    }

    void provideProduct()
    {
        clearAll();
        _requested.append( IEvent::PRODUCT_PROVIDED );
        bSync( _requested, _watched, _blocked, "Provide product" );
    }
};

#endif // __Dispenser_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
