#ifndef __KeyPad_h__
#define __KeyPad_h__

#include "BThread.h"
#include "IEvent.h"
#include "Selector.h"
#include "ServerSocket.h"
#include "Socket.h"
#include "String.h"
#include "T/Truth.h"

class KeyPad : public BThread, public IServerSocket::ISocketAccepter
{
public:
    KeyPad() : BThread( "KeyPad" ), _serverSocket( _selector, 5000, *this ), _socket( NULL )
    {
    }

    void handleAcceptedSocket( ISocket &socket )
    {
        _socket = &socket;
    }

    void entryPoint()
    {
        waitForConnection();

        HeapData request;

        while ( T::alwaysTrue() )
        {
            readAction();
            clearAll();

            if ( _action == "coin" )
            {
                _requested.append( IEvent::COIN_INSERTED );
                bSync( _requested, _watched, _blocked, "Coin" );
            }
            else if ( _action == "drink" )
            {
                _requested.append( IEvent::PRODUCT_CHOSEN );
                bSync( _requested, _watched, _blocked, "Chosen" );
            }
        }
    }

private:
    Vector<Event> _requested;
    Vector<Event> _watched;
    Vector<Event> _blocked;

    Selector _selector;
    ServerSocket _serverSocket;
    ISocket *_socket;
    String _action;

    void clearAll()
    {
        _requested.clear();
        _watched.clear();
        _blocked.clear();
    }

    void waitForConnection()
    {
        _selector.select();
    }

    void readAction()
    {
        HeapData actionData;
        _socket->receive( actionData );

        _action = String( (char *)actionData.data(), actionData.size() - 1 );
    }
};

#endif // __KeyPad_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
