#ifndef __MeasurerStateGraph_h__
#define __MeasurerStateGraph_h__

#include "IEvent.h"
#include "ThreadStateGraph.h"

class MeasurerStateGraph
{
public:
    static ThreadStateGraph stateGraph()
    {
        ThreadStateGraph measurerStateGraph;

        measurerStateGraph._events.append( IEvent::INITIATE_MAINTENANCE );
        measurerStateGraph._events.append( IEvent::TEMPERATURE_HIGH );
        measurerStateGraph._events.append( IEvent::TEMPERATURE_LOW );
        measurerStateGraph._events.append( IEvent::TEMPERATURE_ADJUSTED );
        measurerStateGraph._events.append( IEvent::HUMIDITY_HIGH );
        measurerStateGraph._events.append( IEvent::HUMIDITY_LOW );
        measurerStateGraph._events.append( IEvent::HUMIDITY_ADJUSTED );
        measurerStateGraph._events.append( IEvent::COIN_INSERTED );
        measurerStateGraph._events.append( IEvent::PRODUCT_CHOSEN );
        measurerStateGraph._events.append( IEvent::PRODUCT_PROVIDED );

        measurerStateGraph._initialState = "waitForMaintenance";

        ThreadState waitForMaintenance;

        ThreadState temperatureLow;
        ThreadState temperatureHigh;
        ThreadState temperatureOk;

        ThreadState humidityLow;
        ThreadState humidityHigh;
        ThreadState humidityOk;

        waitForMaintenance._label = "waitForMaintenance";
        waitForMaintenance._watchedEvents.append( IEvent::INITIATE_MAINTENANCE );
        waitForMaintenance._successors.append( Transition( IEvent::INITIATE_MAINTENANCE, "temperatureLow" ) );
        waitForMaintenance._successors.append( Transition( IEvent::INITIATE_MAINTENANCE, "temperatureHigh" ) );
        measurerStateGraph._graph[waitForMaintenance._label] = waitForMaintenance;

        temperatureLow._label = "temperatureLow";
        temperatureLow._requestedEvents.append( IEvent::TEMPERATURE_LOW );
        temperatureLow._watchedEvents.append( IEvent::TEMPERATURE_LOW );
        temperatureLow._watchedEvents.append( IEvent::TEMPERATURE_HIGH );
        temperatureLow._blockedEvents.append( IEvent::PRODUCT_PROVIDED );
        temperatureLow._successors.append( Transition( IEvent::TEMPERATURE_LOW, "temperatureOk" ) );
        measurerStateGraph._graph[temperatureLow._label] = temperatureLow;

        temperatureHigh._label = "temperatureHigh";
        temperatureHigh._requestedEvents.append( IEvent::TEMPERATURE_HIGH );
        temperatureHigh._watchedEvents.append( IEvent::TEMPERATURE_LOW );
        temperatureHigh._watchedEvents.append( IEvent::TEMPERATURE_HIGH );
        temperatureHigh._blockedEvents.append( IEvent::PRODUCT_PROVIDED );
        temperatureHigh._successors.append( Transition( IEvent::TEMPERATURE_HIGH, "temperatureOk" ) );
        measurerStateGraph._graph[temperatureHigh._label] = temperatureHigh;

        temperatureOk._label = "temperatureOk";
        temperatureOk._watchedEvents.append( IEvent::TEMPERATURE_ADJUSTED );
        temperatureOk._blockedEvents.append( IEvent::PRODUCT_PROVIDED );
        temperatureOk._successors.append( Transition( IEvent::TEMPERATURE_ADJUSTED, "humidityLow" ) );
        temperatureOk._successors.append( Transition( IEvent::TEMPERATURE_ADJUSTED, "humidityHigh" ) );
        measurerStateGraph._graph[temperatureOk._label] = temperatureOk;

        humidityLow._label = "humidityLow";
        humidityLow._requestedEvents.append( IEvent::HUMIDITY_LOW );
        humidityLow._watchedEvents.append( IEvent::HUMIDITY_LOW );
        humidityLow._watchedEvents.append( IEvent::HUMIDITY_HIGH );
        humidityLow._successors.append( Transition( IEvent::HUMIDITY_LOW, "humidityOk" ) );
        measurerStateGraph._graph[humidityLow._label] = humidityLow;

        humidityHigh._label = "humidityHigh";
        humidityHigh._requestedEvents.append( IEvent::HUMIDITY_HIGH );
        humidityHigh._watchedEvents.append( IEvent::HUMIDITY_LOW );
        humidityHigh._watchedEvents.append( IEvent::HUMIDITY_HIGH );
        humidityHigh._successors.append( Transition( IEvent::HUMIDITY_HIGH, "humidityOk" ) );
        measurerStateGraph._graph[humidityHigh._label] = humidityHigh;

        humidityOk._label = "humidityOk";
        humidityOk._watchedEvents.append( IEvent::HUMIDITY_ADJUSTED );
        humidityOk._successors.append( Transition( IEvent::HUMIDITY_ADJUSTED, "waitForMaintenance" ) );
        measurerStateGraph._graph[humidityOk._label] = humidityOk;

        return measurerStateGraph;
    }
};

#endif // __MeasurerStateGraph_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//


