#ifndef __Speaker_h__
#define __Speaker_h__

#include "BThread.h"
#include "IEvent.h"
#include "Stringf.h"
#include "T/Truth.h"

class Speaker : public BThread
{
public:
    Speaker() : BThread( "Speaker" )
    {
    }

    void entryPoint()
    {
        _watched = IEvent::allEvents();

        print( "Starting" );

        while ( T::alwaysTrue() )
        {
            bSync( _requested, _watched, _blocked, "Speaker" );

            switch ( lastEvent().type() )
            {
            case IEvent::INITIATE_MAINTENANCE:
                printf( "Initiate Maintenance\n" );
                break;
            case IEvent::TEMPERATURE_HIGH:
                printf( "Temperature High\n" );
                break;
            case IEvent::TEMPERATURE_LOW:
                printf( "Temperature Low\n" );
                break;
            case IEvent::TEMPERATURE_ADJUSTED:
                printf( "Temperature Adjusted\n" );
                break;
            case IEvent::HUMIDITY_HIGH:
                printf( "Humidity High\n" );
                break;
            case IEvent::HUMIDITY_LOW:
                printf( "Humidity Low\n" );
                break;
            case IEvent::HUMIDITY_ADJUSTED:
                printf( "Humidity Adjusted\n" );
                break;
            case IEvent::COIN_INSERTED:
                printf( "Coin Inserted\n" );
                break;
            case IEvent::PRODUCT_CHOSEN:
                printf( "Product Chosen\n" );
                break;
            case IEvent::PRODUCT_PROVIDED:
                printf( "Product Provided\n" );
                break;
            default:
                break;
            }
        }
    }

private:
    Vector<Event> _requested;
    Vector<Event> _watched;
    Vector<Event> _blocked;

    void print( String line )
    {
        printf( "%s\t%s\n", timeString().ascii(), line.ascii() );
    }

    String timeString()
    {
        time_t timeSinceEpoch = time( NULL );

        struct tm *localTime = localtime( &timeSinceEpoch );

        return Stringf( "%0.2d:%0.2d:%0.2d", localTime->tm_hour, localTime->tm_min, localTime->tm_sec );
    }
};

#endif // __Speaker_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
